@@ -1,9 +1,9 @@
 import sys
 
 import core
-from core import _o, o, launch, Return, InvalidYieldException
+from core import _o, o, launch, Return, InvalidYieldException, log_exception
 
-VERSION = '0.16'
+VERSION = '0.17'
 
 _stack_name = None
 def init(stack_name):
